#include "metaformat.h"

#include <fstream>

#include "OIDtools.h"

metaformat::metaformat()
{
}

text_t metaformat::get_mapping(const text_t &collection, const text_t &collectionField)
{
  if (this->oaiConfigure == NULL) {
    return "";
  }

  return this->oaiConfigure->getMapping(collection, collectionField, this->formatPrefix());
}

void metaformat::output_item(ostream &output, bool &headerDone, const text_t &label, 
			     const text_tarray &values)
{ 
  if (!headerDone && (values.size() > 0)) {
    this->output_metadata_header(output);
    headerDone = true;
  }

  for (int item = 0; item < values.size(); ++item) {
    if (this->oaiConfigure->getOAIVersion() >= 200) { // TODO: GRB: This code may need to be subclassed by dc for 200 and later...
      output << "        <" << this->formatPrefix() << ":" << label << ">" << values[item] << "</" << this->formatPrefix() << ":" << label << ">\n";
    }
    else {
      output << "        <" << label << ">" << values[item] << "</" << label << ">\n";
    }
  }
}

bool metaformat::scan_metadata(ostream &output, const text_t &collection, ResultDocInfo_t &docInfo,
			       bool doOutput) 
{ 
  bool headerDone = false;
  MetadataInfo_tmap::iterator here = docInfo.metadata.begin();
  MetadataInfo_tmap::iterator end = docInfo.metadata.end();

  // metaItem is used initially to identify the rfc1807 (etc) metadata items. It is 
  // then used to hold the name of the metadata item, such as "title" or "subject".
  text_t metaItem;
  text_t::const_iterator start, last; // Use two iterators to go through metaItem
  
  while (here != end) {
    start = last = here->first.begin();

    if (here->first.size() < this->formatPrefix().size() ||
	here->first[this->formatPrefix().size()] != '.') {
      metaItem == "";
    }
    else {
      last += this->formatPrefix().size() + 1; // Move last so that it is one place beyond where the "." should be.
      metaItem = substr(start, last);          // Gets the substring starting at start and going up to (but
                                               // not including) last. This should be "dc." (for example)
    }

    if (metaItem == this->formatPrefix()) {
      metaItem = substr(last, here->first.end()); // Get the rest of the metadata tag (it's name)
      lc(metaItem); // Convert it to lowercase for putting in the xml tags

      if (doOutput) {
	this->output_item(output, headerDone, metaItem, here->second.values);
      }
      else {
	if (here->second.values.size() > 0) {
	  return true;
	}
      }
    }  
    else {
      text_t mapTo = this->get_mapping(collection, here->first);
      if (mapTo != "") {
	// Do we actually want to do anything here? Doesn't getting here imply that this 
	// particular metadata is stuff we don't want?
	if (doOutput) {
	  this->output_item(output, headerDone, mapTo, here->second.values);
	}
	else {
	  if (here->second.values.size() > 0) {
	    return true;
	  }
	}
      }
    }

    ++here;
  }

  if (!doOutput) {
    return false;
  }
  
  if (headerDone) {
    this->output_metadata_footer(output);
  }
  
  return headerDone;
}


bool metaformat::is_available(const text_t &collection, ResultDocInfo_t &docInfo)
{ 
  ofstream o("dummy", ios::out);
  return this->scan_metadata(o, collection, docInfo, false);
}

bool metaformat::output_metadata(ostream &output, const text_t &collection, ResultDocInfo_t &docInfo) 
{ 
  return this->scan_metadata(output, collection, docInfo, true);
}

bool metaformat::output_record(ostream &output, recptproto *protocol, const text_t &collection, 
			       const text_t &OID) 
{
  FilterResponse_t response;
  text_tset        metadata;
  ofstream         logout("oai.log", ios::app);

  // get the document information
  if (!get_info(OID, collection, "", metadata, false, protocol, response, logout)) {
    // TODO: error, bad request
    //   cerr << "Bad identifier or protocol " << OID << endl;
    return false;
  }

  // check to see if it's a classifier
  text_t childHead;
  //  int oaiVersion = this->oaiConfigure->getOAIVersion();
  text_t::const_iterator start = OID.begin();
  text_t::const_iterator here  = OID.begin();
  here += 4;
  childHead = substr(start, here);

  // if it isn't a document, kill it now
  if (childHead != "HASH") {
    //    cerr << "Not a document" << endl;
    return false;
  }

  // output record header
  output << "<record>\n";

  // output header part of oai response
  output << "<header>" << endl;
  output << "  <identifier>" << OID << "</identifier>" << endl;
  // TODO: add modified date

  output << "</header>" << endl;

  // output metadata part of oai response
  this->output_metadata(output, collection, response.docInfo[0]);

  // output the description of the document
  //  output << "<about>\n";
  //  output << "</about>\n";

  // close record
  output << "</record>\n";
    
  return true;
}
